#!/usr/bin/env python

# Simple script to export a list of connected clients for the given SSID.
#
# @author Christoph Bless
#
from __future__ import print_function

import argparse
import json
import sqlite3
import sys

from kismetanalyzer.model import AccessPoint
from kismetanalyzer.util import does_ssid_matches


def gen_clientlist():
    parser = argparse.ArgumentParser(description="Print a list of connected clients for the given SSID.")
    parser.add_argument("--in", action="store", dest="infile", required=True, help='Input file (.kismet)')
    parser.add_argument("--ssid", action="store", dest="ssid", required=True,
                        help='SSID (or SSID regex)')
    parameters = parser.parse_args()


    try:
        db = sqlite3.connect(parameters.infile)
    except Exception as e:
        print ("Failed to open kismet logfile: {0}".format(e))
        sys.exit(1)

    try:
        sql = "SELECT * FROM devices where type='Wi-Fi AP'; "
        c = db.cursor()
        sql_result = c.execute(sql)
    except:
        print ("Failed to extract data from database")
        sys.exit()

    # container for collecting relevant devices
    devs = set()

    for row in sql_result:
        try:
            # create a device dictionary from json string stored in the
            # device column of the kismet database
            dev = json.loads(row[14])

            # convert json device string into an instance of the
            # class kismetanalyzer.model.AccessPoint
            ap = AccessPoint.from_json(dev)

            # Apply SSID filter if it is used as parameter (this switch
            # checks the included SSID list, which is provided by the
            # parameter --ssid
            if parameters.ssid is not None:
                if not does_ssid_matches(dev, parameters.ssid):
                    # SSID doesn't match, skip this access point
                    continue

            client_map = ap.client_map
            for c in client_map:
                devs.add(c)

        except Exception as e:
            continue

    print("\n".join(devs))



#!/usr/bin/env python

# Simple script to export a list of access points to either csv and / or 
# to kml.
#
# @author Christoph Bless
# 
from __future__ import print_function

import argparse
import json
import sqlite3
import sys

from fastkml import kml, styles
from pygeoif import geometry

from kismetanalyzer.model import AccessPoint
from kismetanalyzer.util import does_ssid_matches


def get_description(ap):
    """
    This fuction is used to create the description string for the device.
    
    :param ap: instance of kismetanalzer.model.AccessPoint class
    
    :return: A string which can be used as description for the device
    :rtype string
    """
    clients = "\n".join(ap.client_map)
    desc = "MAC: {0}\nEncryption: {1}\nFrequency: {2}\nChannel: {3}\nManufacturer: {4}\n\nClients:\n{5}"
    description = desc.format(ap.mac, ap.encryption, ap.frequency, ap.channel, ap.manufacturer, clients)
    return description
    
    
def get_networkcolor(encryption):
    """
    This fuction is used to get color for a network which will be added 
    to a KML-File.s
    
    :param encryption: encryption string 
    
    :return: Color to use for the network
    :rtype simplekml.Color
    """
    if 'WPA' in encryption:
        # green
        return "ff008000"
    elif 'WEP' in encryption:
        # orange
        return "ff00a5ff"
    elif 'Open' in encryption: 
        # red
        return "ff0000ff"
    else:
        # yellow
        return "ff00ffff"


def export_csv(filename, devices, delimiter=";"):
    """
    Export found devices to a CSV file. The filename prefix and the list 
    of devices is required. The delimiter is optional.
    
    :param filename: Prefix for the filename. The extention "csv" will be added
    :param devices: list of devices. Each device must be a tuple with 
                    the followin format (lon, lat, mac, title, encryption, description )
    :param delimiter: Delimiter to use for separation of columns (optional)
    """
    import csv
    
    num_plotted = 0
    
    outfile = "{0}-aplist.csv".format(filename)
    
    with open(outfile, mode='w') as csv_file:
        w = csv.writer(csv_file, delimiter=delimiter, quotechar='"', quoting=csv.QUOTE_MINIMAL)
        w.writerow(['MAC-Address', 'SSID', 'Encryption', 'Frequency', 'Channel', 'Manufacturer'])
        for dev in devices:
            w.writerow([dev.mac, dev.ssid, dev.encryption, dev.frequency, dev.channel, dev.manufacturer])
            num_plotted = num_plotted + 1
        
    print ("Exported {} devices to {}".format(num_plotted, outfile))


def export_kml(filename, title, devices):
    """
    Export found devices to a KML file which can be imported to Googleearth.
    The filename prefix and the list of devices is required.

    :param filename: Prefix for the filename. The extention "kml" will be added
    :param title: name which will be added to kml file
    :param devices: list of devices. Each device must be a tuple in the following format (lon, lat, mac, title, encryption, description )
   """
    num_plotted = 0
    outfile = "{0}-aplist.kml".format(filename)

    # create a KML file skeleton
    k = kml.KML()
    ns = '{http://www.opengis.net/kml/2.2}'

    doc = kml.Document(ns, "docid", title, '')
    k.append(doc)

    # create placemark for the access point, and add it to the KML document
    for dev in devices:
        icon_style = styles.IconStyle(ns=ns, color=get_networkcolor(dev.encryption))
        style = styles.Style(ns=ns, styles=[icon_style])
        desc = get_description(dev)
        p = kml.Placemark(name=dev.ssid, description=desc, styles=[style])
        p.geometry = geometry.Point(dev.location.lon, dev.location.lat, dev.location.alt)
        doc.append(p)
        num_plotted = num_plotted + 1

    with open (outfile, "w") as f:
        s = str(k.to_string(prettyprint=True))
        f.write(s)

    print("Exported {} devices to {}".format(num_plotted, outfile))


def gen_aplist():
    parser = argparse.ArgumentParser(description="List access points discovered by kismet.")
    parser.add_argument("--in", action="store", dest="infile", required=True, help='Input file (.kismet)')
    parser.add_argument("--out", action="store", dest="outfile", help='Output filename (optional)')
    parser.add_argument("--title", action="store", dest="title", default="Kismet", help='Title embedded in KML file')
    parser.add_argument("--ssid", action="store", dest="ssid", help='Only plot networks which match the SSID (or SSID regex)')
    parser.add_argument("--exclude-ssid", action="store", dest="excludessid", help='Exclude networks which match the SSID (or SSID regex)')
    parser.add_argument("--strongest-point", action="store_true", dest="strongest", default=False, help='Plot points based on strongest signal')
    parser.add_argument("--encryption", action="store", dest="encryption", default=None, help="Show only networks with given encryption type" )
    parser.add_argument("--csv", action="store_true", dest="csv", default=False, help="Export results to csv")
    parser.add_argument("--kml", action="store_true", dest="kml", default=False, help="Export results to kml")
    parser.add_argument("--verbose", action="store_true", dest="verbose", default=False, help="Print MAC, SSID, encryption type to stdout")
    parameters = parser.parse_args()

    # set the filename prefix for the output file if it is not specified
    # via the parameter --out
    if parameters.outfile is None:
        if parameters.infile.endswith(".kismet"):
            parameters.outfile = parameters.infile[:-7]
        else:
            parameters.outfile = parameters.infile
    
    try:
        db = sqlite3.connect(parameters.infile)
    except Exception as e:
        print ("Failed to open kismet logfile: {0}".format(e))
        sys.exit(1)
    
    try:  
        sql = "SELECT * FROM devices where type='Wi-Fi AP'; "
        c = db.cursor()
        sql_result = c.execute(sql)    
    except:
        print ("Failed to extract data from database")
        sys.exit()
    
    # container for collecting relevant devices
    devs = []
    for row in sql_result:
        try:
            # create a device dictionary from json string stored in the
            # device column of the kismet database
            dev = json.loads(row[14])
            
            # convert json device string into an instance of the 
            # class kismetanalyzer.model.AccessPoint
            strongest = parameters.strongest
            ap = AccessPoint.from_json(dev, strongest)
            
            # Apply SSID filter if it is used as parameter (this switch
            # checks the included SSID list, which is provided by the 
            # parameter --ssid 
            if parameters.ssid is not None:
                if not does_ssid_matches(dev, parameters.ssid):
                    # SSID doesn't match, skip this access point
                    continue
            
            # Apply SSID filter if it is used as parameter (this switch 
            # checks the excluded SSID list, which is provided by the 
            # parameter --exclude-ssid )
            if parameters.excludessid is not None:
                if does_ssid_matches(dev, parameters.excludessid):
                    # SSID matches, skip this access point
                    continue
            
            # skip device if the secified encryption string is not 
            # present in the device encryption string
            if parameters.encryption:
                if not parameters.encryption in ap.encryption:
                    continue


            if parameters.verbose:
                print ("{:20s}{:20s}{:40s}".format(ap.mac, ap.encryption, ap.ssid))

            devs.append(ap)
        
        except Exception as e:
            continue

    
    if parameters.csv:
        export_csv(parameters.outfile, devs)
         
    if parameters.kml:
        export_kml(parameters.outfile, parameters.title, devs)




# This script contains some functions to parse the json strings form the 
# kismet database column "device" of table "devices"

import re


def parse_clientmap(dev):
    clients = []
    if 'dot11.device' in dev:
        if "dot11.device.associated_client_map" in dev['dot11.device']:
            temp = dev['dot11.device']["dot11.device.associated_client_map"]
            for k in temp:
                # the key contains the client MAC address. only this is added to the client list
                clients.append(k)
    return clients


def parse_networkname(dev):
    """
    This function is used to parse the network name (SSID) from the json
    string, which is written to the device column of the kismet database.
    
    :param dev: json string from the kismet database column "device"
    
    :return: A string either with the network name or MAC-Address if the SSID is not available
    :rtype string
    """
    netname = ""

    if 'kismet.device.base.name' in dev:
        netname = dev['kismet.device.base.name']
    elif 'dot11.device' in dev:
            if 'dot11.device.advertised_ssid_map' in dev['dot11.device']:
                ssid_map = dev['dot11.device']['dot11.device.advertised_ssid_map']
                if 'dot11.advertisedssid.ssid' in ssid_map:
                    netname = ssid_map['dot11.advertisedssid.ssid']
            elif 'dot11.device.last_beaconed_ssid' in dev['dot11.device']:
                netname = dev['dot11.device']['dot11.device.last_beaconed_ssid']
    else:
        netname = dev['kismet.device.base.macaddr']
    
    return netname
    

def parse_loc(dev, strongest=False):
    """
    This function is used to extract the longitude and latitude from a
    result set of an SQL query of the kismet database table devices. 
    
    :param row: SQL result set 
    
    :return: A tuple with longitude, latitude, altitude (long, lat, alt)
    :rtype tuple
    """
    loc = None
    lon = "0" 
    lat = "0"
    alt = "0"
    if 'kismet.device.base.location' in dev:
        if strongest:
            # strongest location
            if 'kismet.common.location.max_loc' in dev['kismet.device.base.location']:
                loc = dev['kismet.device.base.location']['kismet.common.location.max_loc']
                if 'kismet.common.location.lon' in loc \
                        and 'kismet.common.location.lat' in loc \
                        and 'kismet.common.location.alt' in loc:
                    lon = loc['kismet.common.location.lon']
                    lat = loc['kismet.common.location.lat']
                    alt = loc['kismet.common.location.alt']
                elif 'kismet.common.location.geopoint' in loc:
                    geopoint = loc['kismet.common.location.geopoint']
                    lat = geopoint[0]
                    lon = geopoint[1]
        else:
            # average location
            if 'kismet.common.location.avg_loc' in dev['kismet.device.base.location']:
                loc =  dev['kismet.device.base.location']['kismet.common.location.avg_loc']
                if 'kismet.common.location.lon' in loc \
                        and 'kismet.common.location.lat' in loc \
                        and 'kismet.common.location.alt' in loc:
                    lon = loc['kismet.common.location.lon']
                    lat = loc['kismet.common.location.lat']
                    alt = loc['kismet.common.location.alt']
                elif 'kismet.common.location.geopoint' in loc:
                    geopoint = loc['kismet.common.location.geopoint']
                    lat = geopoint[0]
                    lon = geopoint[1]
    
    return (lon, lat, alt)
    
def parse_mac(dev):
    """
    This function is used to parse the MAC-Address from the json string,
    which is written to the device column of the kismet database.
    
    :param dev: json string from the kismet database column "device"
    
    :return: MAC-Address as string
    :rtype string
    """
    if 'kismet.device.base.macaddr' in dev:
        return dev['kismet.device.base.macaddr']
    return ""
    

def parse_encryption(dev):
    """
    This function is used to parse the encryption type from the json string,
    which is written to the device column of the kismet database.
    
    :param dev: json string from the kismet database column "device"
    
    :return: Encryption type as string
    :rtype: string
    """
    if 'kismet.device.base.crypt' in dev:
        return dev['kismet.device.base.crypt']
    return ""
    

def parse_frequency(dev):
    """
    This function is used to extract the frequency from the json string, 
    which is written to the device column of the kismet database.
    
    :param dev: json string from the kismet database column "device"
    
    :retrurn: Frequency as string
    :rtype: string
    """
    if 'kismet.device.base.frequency' in dev:
        return dev['kismet.device.base.frequency']
    return ""
    
    
def parse_channel(dev):
    """
    This function is used to extract the channel from the json string, 
    which is written to the device column of the kismet database.
    
    :param dev: json string from the kismet database column "device"
    
    :retrurn: Channelnumber as string
    :rtype: string
    """
    if 'kismet.device.base.channel' in dev:
        return dev['kismet.device.base.channel']
    return ""
    

def parse_manufacturer(dev):
    """
    This function is used to extract the manufacturer name from the json 
    string, which is written to the device column of the kismet database.
    
    :param dev: json string from the kismet database column "device"
    
    :retrurn: Manufacturer name as string
    :rtype: string
    """
    if 'kismet.device.base.manuf' in dev:
        return dev['kismet.device.base.manuf']
    return ""


def parse_name(dev):
    """
    This function is used to parse the name from the json string,
    which is written to the device column of the kismet database.

    :param dev: json string from the kismet database column "device"

    :return: Encryption type as string
    :rtype: string
    """
    if 'kismet.device.base.name' in dev:
        return dev['kismet.device.base.name']
    return ""


def parse_commonname(dev):
    """
    This function is used to parse the "commonname" from the json string,
    which is written to the device column of the kismet database.

    :param dev: json string from the kismet database column "device"

    :return: Encryption type as string
    :rtype: string
    """
    if 'kismet.device.base.commonname' in dev:
        return dev['kismet.device.base.commonname']
    return ""


def parse_phyname(dev):
    """
    This function is used to parse the "phyname" from the json string,
    which is written to the device column of the kismet database.

    :param dev: json string from the kismet database column "device"

    :return: Encryption type as string
    :rtype: string
    """
    if 'kismet.device.base.phyname' in dev:
        return dev['kismet.device.base.phyname']
    return ""


def parse_type(dev):
    """
    This function is used to parse the "type" from the json string,
    which is written to the device column of the kismet database.

    :param dev: json string from the kismet database column "device"

    :return: Encryption type as string
    :rtype: string
    """
    if 'kismet.device.base.type' in dev:
        return dev['kismet.device.base.type']
    return ""


def does_ssid_matches(dev, ssid):
    """
    checks if the device SSID matches the given SSID string.
    
    :param dev: json string from the kismet database column "device"
    :param ssid: SSID string or regex 
    
    :return : true if the SSID matches, false otherwise
    :rtype: boolean
    """
    matched = False
    if 'kismet.device.base.name' in dev:
        if re.match(ssid, dev['kismet.device.base.name']):
            matched = True
    return matched



#!/usr/bin/env python

# Simple script to export a list of access points to either csv and / or
# to kml.
#
# @author Christoph Bless
#
from __future__ import print_function

import argparse
import json
import sqlite3
import sys

from fastkml import kml, styles
from pygeoif import geometry

from kismetanalyzer.model import Device
from kismetanalyzer.util import does_ssid_matches


def get_description(dev):
    """
    This function is used to create the description string for the device.

    :param ap: instance of kismetanalzer.model.Device class

    :return: A string which can be used as description for the device
    :rtype string
    """
    desc = "MAC: {0}\nName: {1}\nCommonname: {2}\nPhyName: {3}\nManufacturer: {4}\nType: {5}\nFrequency: {6}\nChannel: {7}"
    description = desc.format(dev.mac, dev.name, dev.commonname, dev.phyname, dev.manufacturer, dev.type, dev.frequency, dev.channel)
    return description



def export_csv(filename, devices, delimiter=";"):
    """
    Export found devices to a CSV file. The filename prefix and the list
    of devices is required. The delimiter is optional.

    :param filename: Prefix for the filename. The extention "csv" will be added
    :param devices: list of devices. Each device must be a tuple with
                    the following format (lon, lat, mac, title, encryption, description )
    :param delimiter: Delimiter to use for separation of columns (optional)
    """
    import csv

    num_plotted = 0

    outfile = "{0}-devices.csv".format(filename)

    with open(outfile, mode='w') as csv_file:
        w = csv.writer(csv_file, delimiter=delimiter, quotechar='"', quoting=csv.QUOTE_MINIMAL)
        w.writerow(['MAC-Address', 'TYPE', 'NAME', 'COMMONNAME', 'PHYNAME', 'Frequency', 'Channel', 'Manufacturer'])
        for dev in devices:
            w.writerow([dev.mac, dev.type, dev.name, dev.commonname, dev.phyname, dev.frequency, dev.channel, dev.manufacturer])
            num_plotted = num_plotted + 1

    print("Exported {} devices to {}".format(num_plotted, outfile))


def export_kml(filename, title, devices):
    """
    Export found devices to a KML file which can be imported to Googleearth.
    The filename prefix and the list of devices is required.

    :param filename: Prefix for the filename. The extention "kml" will be added
    :param title: name which will be added to kml file
    :param devices: list of devices. Each device must be a tuple in the following format (lon, lat, mac, title, encryption, description )
   """
    num_plotted = 0
    outfile = "{0}-devices.kml".format(filename)

    # create a KML file skeleton
    k = kml.KML()
    ns = '{http://www.opengis.net/kml/2.2}'

    doc = kml.Document(ns, "docid", title, '')
    k.append(doc)

    # create placemark for the access point, and add it to the KML document
    for dev in devices:
        icon_style = styles.IconStyle(ns=ns, color="ff00ffff")
        style = styles.Style(ns=ns, styles=[icon_style])
        desc = get_description(dev)
        p = kml.Placemark(name=dev.name, description=desc, styles=[style])
        p.geometry = geometry.Point(dev.location.lon, dev.location.lat, dev.location.alt)
        doc.append(p)
        num_plotted = num_plotted + 1

    with open(outfile, "w") as f:
        s = str(k.to_string(prettyprint=True))
        f.write(s)

    print("Exported {} devices to {}".format(num_plotted, outfile))


def gen_devlist():
    parser = argparse.ArgumentParser(description="List devices discovered by kismet.")
    parser.add_argument("--in", action="store", dest="infile", required=True, help='Input file (.kismet)')
    parser.add_argument("--out", action="store", dest="outfile", help='Output filename (optional)')
    parser.add_argument("--title", action="store", dest="title", default="Kismet", help='Title embedded in KML file')
    parser.add_argument("--csv", action="store_true", dest="csv", default=False, help="Export results to csv")
    parser.add_argument("--kml", action="store_true", dest="kml", default=False, help="Export results to kml")
    parser.add_argument("--strongest-point", action="store_true", dest="strongest", default=False,
                        help='Plot points based on strongest signal')
    parser.add_argument("--type", action="store", dest="type", default=None, help='Filter by Type')
    parser.add_argument("--verbose", action="store_true", dest="verbose", default=False,
                        help="Print MAC, TYPE, CHANNEL type to stdout")
    parameters = parser.parse_args()

    # set the filename prefix for the output file if it is not specified
    # via the parameter --out
    if parameters.outfile is None:
        if parameters.infile.endswith(".kismet"):
            parameters.outfile = parameters.infile[:-7]
        else:
            parameters.outfile = parameters.infile

    try:
        db = sqlite3.connect(parameters.infile)
    except Exception as e:
        print("Failed to open kismet logfile: {0}".format(e))
        sys.exit(1)

    try:
        sql = "SELECT * FROM devices; "
        c = db.cursor()
        sql_result = c.execute(sql)
    except:
        print("Failed to extract data from database")
        sys.exit()

    # container for collecting relevant devices
    devs = []
    for row in sql_result:
        try:
            # create a device dictionary from json string stored in the
            # device column of the kismet database
            dev = json.loads(row[14])
            # convert json device string into an instance of the
            # class kismetanalyzer.model.AccessPoint
            strongest = parameters.strongest
            d = Device.from_json(dev, strongest)
            if parameters.type is not None:
                if parameters.type not in d.type:
                    continue

            if parameters.verbose:
                print("{:20s}{:40s}{:10s}".format(d.mac, d.type, d.channel))

            devs.append(d)

        except Exception as e:
            continue

    if parameters.csv:
        export_csv(parameters.outfile, devs)

    if parameters.kml:
        export_kml(parameters.outfile, parameters.title, devs)



# -*- coding: utf-8 -*-
from setuptools import setup
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='kismet-analyzer',
    version='0.4.0',
    author='Christoph Bless',
    author_email='bitbucket@cbless.de',
    url='https://bitbucket.org/cbless/kismetanalyzer',
    license=' GPLv3',
    description=('Library for parsing kismet results from the .kismet database file.'),
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['kismetanalyzer'],
    install_requires=[
        'fastkml',
        'pygeoif',
        'lxml'
    ],
    entry_points = {
        "console_scripts": [
            "kismet_analyzer_aplist = kismetanalyzer.aplist:gen_aplist",
            "kismet_analyzer_clientlist = kismetanalyzer.clientlist:gen_clientlist",
            "kismet_analyzer_devices = kismetanalyzer.devices:gen_devlist",
        ]
    }
)
